/*******************************************************************************
 * Copyright (c) 2013-2015 Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v3
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * Contributors:
 *     Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza - initial API and implementation
 ******************************************************************************/
package com.net2plan.examples.onlineSim.cac;

import com.net2plan.interfaces.networkDesign.Net2PlanException;
import com.net2plan.interfaces.networkDesign.NetPlan;
import com.net2plan.interfaces.simulation.IEventProcessor;
import com.net2plan.interfaces.simulation.SimAction;
import com.net2plan.interfaces.simulation.SimEvent;
import com.net2plan.libraries.GraphUtils;
import com.net2plan.utils.DoubleUtils;
import com.net2plan.utils.Pair;
import com.net2plan.utils.Triple;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * Basic CAC algorithm which tries to route each connection request in the shortest path.
 *
 * @author Pablo Pavon-Marino, Jose-Luis Izquierdo-Zaragoza
 * @version 1.1, May 2015
 */
public class CACSim_AA_basicAlgorithm extends IEventProcessor
{
	private Map<Long, Double> costMap;
	private Map<Long, Pair<Long, Long>> linkMap;

	@Override
	public String getDescription()
	{
		return "Basic CAC algorithm which tries to route each connection request in the shortest path";
	}

	@Override
	public List<Triple<String, String, String>> getParameters()
	{
		final List<Triple<String, String, String>> algorithmParameters = new LinkedList<Triple<String, String, String>>();
		algorithmParameters.add(Triple.of("shortestPathType", "#select# hops km", "Criteria to compute the shortest path. Valid values: 'hops' or 'km'"));

		return algorithmParameters;
	}

	@Override
	public void initialize(NetPlan initialNetPlan, NetPlan currentNetPlan, Map<String, String> algorithmParameters, Map<String, String> simulationParameters, Map<String, String> net2planParameters)
	{
		final String shortestPathType = algorithmParameters.get("shortestPathType");
		if (!shortestPathType.equalsIgnoreCase("km") && !shortestPathType.equalsIgnoreCase("hops"))
			throw new Net2PlanException("Wrong 'shortestPathType' parameter");
		
		Set<Long> linkIds = initialNetPlan.getLinkIds();
		costMap = shortestPathType.equalsIgnoreCase("hops") ? DoubleUtils.ones(linkIds) : initialNetPlan.getLinkLengthInKmMap();
		linkMap = initialNetPlan.getLinkMap();
	}

	@Override
	public void processEvent(NetPlan currentNetPlan, SimEvent event)
	{
		List<SimAction> actions = event.getEventActionList();
		
		for(SimAction action : actions)
		{
			if (action.getActionType() == SimAction.ActionType.DEMAND_ADDED)
			{
				long demandId = action.getDemandAdded_demandId();

				long ingressNodeId = currentNetPlan.getDemandIngressNode(demandId);
				long egressNodeId = currentNetPlan.getDemandEgressNode(demandId);
				double h_d = currentNetPlan.getDemandOfferedTraffic(demandId);

				List<Long> spLinks = GraphUtils.getShortestPath(linkMap, ingressNodeId, egressNodeId, costMap);
				if (!spLinks.isEmpty()) currentNetPlan.addRoute(demandId, h_d, spLinks, null);
			}
		}
	}
}
